--
-- LevelRange - Shows the zone level range on the World Map.
-- Copyright (c) 2006 Philip Hughes (Bull3t)
-- 
-- Thanks to Dhask for allowing the use of his FlightMap.
--
-- An unlimited license to use, reproduce and copy this work is granted, on
-- the condition that the licensee accepts all responsibility and liability
-- for any damage that may arise from the use of this AddOn.
--

-- Version Number
LEVELRANGE_VERSION    = "1.0.0";

-- Faction Types
local lTYPE_HORDE     = LEVELRANGE_HORDE;
local lTYPE_ALLIANCE  = LEVELRANGE_ALLIANCE;
local lTYPE_CONTESTED = LEVELRANGE_CONTESTED;

-- Level Ranges
LEVELRANGE_RANGES   = {
    [LEVELRANGE_ELWYNN]        = { 1, 10, lTYPE_ALLIANCE},
    [LEVELRANGE_DUNMOROGH]     = { 1, 10, lTYPE_ALLIANCE},
    [LEVELRANGE_TIRISFAL]      = { 1, 10, lTYPE_HORDE},
    [LEVELRANGE_LOCHMODAN]     = {10, 20, lTYPE_ALLIANCE},
    [LEVELRANGE_SILVERPINE]    = {10, 20, lTYPE_HORDE},
    [LEVELRANGE_WESTFALL]      = {10, 20, lTYPE_ALLIANCE},
    [LEVELRANGE_REDRIDGE]      = {15, 25, lTYPE_CONTESTED},
    [LEVELRANGE_DUSKWOOD]      = {18, 30, lTYPE_CONTESTED},
    [LEVELRANGE_HILLSBRAD]     = {20, 30, lTYPE_CONTESTED},
    [LEVELRANGE_WETLANDS]      = {20, 30, lTYPE_CONTESTED},
    [LEVELRANGE_ALTERAC]       = {30, 40, lTYPE_CONTESTED},
    [LEVELRANGE_ARATHI]        = {30, 40, lTYPE_CONTESTED},
    [LEVELRANGE_STRANGLETHORN] = {30, 45, lTYPE_CONTESTED},
    [LEVELRANGE_BADLANDS]      = {35, 45, lTYPE_CONTESTED},
    [LEVELRANGE_SORROWS]       = {35, 45, lTYPE_CONTESTED},
    [LEVELRANGE_HINTERLANDS]   = {40, 50, lTYPE_CONTESTED},
    [LEVELRANGE_SEARINGGORGE]  = {43, 50, lTYPE_CONTESTED},
    [LEVELRANGE_BLASTEDLANDS]  = {45, 55, lTYPE_CONTESTED},
    [LEVELRANGE_BURNINGSTEPPE] = {50, 58, lTYPE_CONTESTED},
    [LEVELRANGE_WESTERNPLAGUE] = {51, 58, lTYPE_CONTESTED},
    [LEVELRANGE_EASTERNPLAGUE] = {53, 60, lTYPE_CONTESTED},
    [LEVELRANGE_DEADWINDPASS]  = {55, 60, lTYPE_CONTESTED},
    
    [LEVELRANGE_DUROTAR]       = { 1, 10, lTYPE_HORDE},
    [LEVELRANGE_MULGORE]       = { 1, 10, lTYPE_HORDE},
    [LEVELRANGE_DARKSHORE]     = {10, 20, lTYPE_ALLIANCE},
    [LEVELRANGE_BARRENS]       = {10, 25, lTYPE_HORDE},
    [LEVELRANGE_STONETALON]    = {15, 27, lTYPE_CONTESTED},
    [LEVELRANGE_ASHENVALE]     = {18, 30, lTYPE_CONTESTED},
    [LEVELRANGE_1KNEEDLES]     = {25, 35, lTYPE_CONTESTED},
    [LEVELRANGE_DESOLACE]      = {30, 40, lTYPE_CONTESTED},
    [LEVELRANGE_DUSTWALLOW]    = {35, 45, lTYPE_CONTESTED},
    [LEVELRANGE_FERALAS]       = {40, 50, lTYPE_CONTESTED},
    [LEVELRANGE_TANARIS]       = {40, 50, lTYPE_CONTESTED},
    [LEVELRANGE_AZSHARA]       = {45, 55, lTYPE_CONTESTED},
    [LEVELRANGE_FELWOOD]       = {48, 55, lTYPE_CONTESTED},
    [LEVELRANGE_UNGOROCRATER]  = {48, 55, lTYPE_CONTESTED},
    [LEVELRANGE_SILITHUS]      = {55, 60, lTYPE_CONTESTED},
    [LEVELRANGE_WINTERSPRING]  = {55, 60, lTYPE_CONTESTED},

    [LEVELRANGE_MOONGLADE]     = { 1, 60, lTYPE_CONTESTED},
    
    [LEVELRANGE_TELDRASSIL]    = { 1, 10, lTYPE_ALLIANCE},

};

-- Sub-zones
LEVELRANGE_SUBZONES = {
    [LEVELRANGE_ORGRIMMAR]    = LEVELRANGE_DUROTAR,
    [LEVELRANGE_THUNDERBLUFF] = LEVELRANGE_MULGORE,
    [LEVELRANGE_UNDERCITY]    = LEVELRANGE_TIRISFAL,
    [LEVELRANGE_IRONFORGE]    = LEVELRANGE_DUNMOROGH,
    [LEVELRANGE_STORMWIND]    = LEVELRANGE_ELWYNN,
};

-- Faction Colours
LEVELRANGE_COLORS = {
    Unknown   = { r = 0.8, g = 0.8, b = 0.8 },
    Hostile   = { r = 0.9, g = 0.2, b = 0.2 },
    Friendly  = { r = 0.2, g = 0.9, b = 0.2 },
    Contested = { r = 0.8, g = 0.6, b = 0.4 },
    
    None 	  = { r = 1, g = 1, b = 1 },
    Levels 	  = { r = 0.8, g = 0.6, b = 0 },
};


------------------ Map Drawing Functions ------------------

-- Update the tooltip for a zone
local function lUpdateTooltip(zoneName)
    -- No zone name, no tooltip!
    if not zoneName or zoneName == "" then
        LevelRangeTooltip:Hide();
        return;
    end

    -- Unused anchor point
    LevelRangeTooltip:SetOwner(this, "ANCHOR_LEFT");

    -- Determine colour and level range
    local title = LEVELRANGE_COLORS.Unknown;
    local normalcol = LEVELRANGE_COLORS.None;
    local levelscol = LEVELRANGE_COLORS.Levels;

    local levels = nil;
    local actualside = nil;

    if (LEVELRANGE_RANGES[zoneName]) then
        local _, faction = UnitFactionGroup("player");
        local min = LEVELRANGE_RANGES[zoneName][1];
        local max = LEVELRANGE_RANGES[zoneName][2];
        local side = LEVELRANGE_RANGES[zoneName][3];
        
        if (side == lTYPE_CONTESTED) then
            title = LEVELRANGE_COLORS.Contested;
            actualside = "Contested";
        else
            if (faction == side) then
                title = LEVELRANGE_COLORS.Friendly;
                actualside = "Friendly";
            else
                title = LEVELRANGE_COLORS.Hostile;
                actualside = "Hostile";
            end
        end
        levels = string.format(LEVELRANGE_LEVELS, min, max);
    end

    -- Show the zone title and add level range if known
    LevelRangeTooltip:SetText(zoneName, normalcol.r, normalcol.g, normalcol.b);
    if levels then
        LevelRangeTooltip:AddLine(levels, levelscol.r, levelscol.g, levelscol.b);
    end
    if actualside then
        LevelRangeTooltip:AddLine(actualside, title.r, title.g, title.b);
    end

    -- This stuff seems to get reset each time
    LevelRangeTooltip:SetBackdropColor(0, 0, 0, 0.5);
    LevelRangeTooltip:SetBackdropBorderColor(1, 1, 1, 1);
    LevelRangeTooltip:ClearAllPoints();
    LevelRangeTooltip:SetPoint("BOTTOMLEFT", "WorldMapDetailFrame", "BOTTOMLEFT", 0, 0);

    -- Only show if there is level information
    if levels then
        LevelRangeTooltip:Show();
    else
        LevelRangeTooltip:Hide();
    end

    -- Now go ahead and put the tooltip into the right location
    LevelRangeTooltip:ClearAllPoints();
    LevelRangeTooltip:SetPoint("BOTTOMLEFT", WorldMapDetailFrame);
end

-- Last drawn info for tooltip
lFM_CurrentZone = nil;
lFM_CurrentArea = nil;
local lFM_OldUpdate = function() end;

-- Replacement function to draw all the extra goodies of LevelRange
function LevelRange_WorldMapButton_OnUpdate(arg1)
    lFM_OldUpdate(arg1);
    local areaName = WorldMapFrame.areaName;
    local zoneNum = GetCurrentMapZone();

    -- zone name equivalence map
    if LEVELRANGE_SUBZONES[areaName] then
        areaName = LEVELRANGE_SUBZONES[areaName];
    end

    -- Bail out if nothing has changed
    if zoneNum == lFM_CurrentZone and areaName == lFM_CurrentArea then
        return;
    end

    -- Continent or zone map?
    if zoneNum == 0 then
        lUpdateTooltip(areaName);
    else
        lUpdateTooltip(nil);            -- hide it
    end
end


------------------ Print Message Function -----------------

function printMSG(msg)
   DEFAULT_CHAT_FRAME:AddMessage("|CF4FFFF4FLevelRange|r: " .. msg,1,1,1);
end


---------------- Initialization Functions -----------------

function LevelRange_OnLoad()
    -- Override the world map function
    lFM_OldUpdate = WorldMapButton_OnUpdate;
    WorldMapButton_OnUpdate = LevelRange_WorldMapButton_OnUpdate;

    -- LevelRange is loaded
    printMSG("LevelRange version " .. LEVELRANGE_VERSION .. " loaded.")
end

function LevelRange_OnEvent(event)
    return;
end